<?php
class BrowserUtils
{
	/**
	 * Detect browser names and versions of Chrome, Firefox, Internet Explorer, Opera & Safari.
	 * 
	 * Returns array('name' => Browser name (as written here ^),
	 * 				 'version' => array(major version, minor subversion, release, build)).
	 * 
	 * 'version' is array of integers. 
	 * 
	 * In case of no browser detected method returns array(null, array()).
	 * 
	 * All non-digital version flags (a, b, beta, pre and so on) are skipped.
	 * If browser version is shorter then 4 parts separated by point (.),
	 * undetected parts will be set in 'version' array as null.
	 * 
	 * This code written with great help of http://www.useragentstring.com/ website
	 * 
	 * The code is actual on 2009-09-15.
	 * 
	 * @author Leontyev Valera (feedbee@gmail.com)
	 * @copyright 2009
	 * @license BSD
	 * 
	 * @param string $userAgent
	 * @return array
	 */
	static function detectBrowser($userAgent = null)
	{
		is_null($userAgent) && ($userAgent = $_SERVER['HTTP_USER_AGENT']);
		$name = null;
		$version = array(null, null, null, null);
		if (false !== strpos($userAgent, 'Opera/'))
		{
			//http://www.useragentstring.com/pages/Opera/
			$name = 'Opera';
			if (false !== strpos($userAgent, 'Version/')) // http://dev.opera.com/articles/view/opera-ua-string-changes/
			{
				preg_match('#Version/(\d{1,2})\.(\d{1,2})#i', $userAgent, $versionMatch);
				isset($versionMatch[1]) && $version[0] = (int)$versionMatch[1];
				isset($versionMatch[2]) && $version[1] = (int)$versionMatch[2];
			}
			else
			{
				preg_match('#Opera/(\d{1,2})\.(\d{1,2})#i', $userAgent, $versionMatch);
				isset($versionMatch[1]) && $version[0] = (int)$versionMatch[1];
				isset($versionMatch[2]) && $version[1] = (int)$versionMatch[2];
			}
		}
		else if (false !== strpos($userAgent, 'Opera '))
		{
			//http://www.useragentstring.com/pages/Opera/
			$name = 'Opera';
			preg_match('#Opera (\d{1,2})\.(\d{1,2})#i', $userAgent, $versionMatch);
			isset($versionMatch[1]) && $version[0] = (int)$versionMatch[1];
			isset($versionMatch[2]) && $version[1] = (int)$versionMatch[2];
		}
		else if (false !== strpos($userAgent, 'Firefox/'))
		{
			// http://www.useragentstring.com/pages/Firefox/
			$name = 'Firefox';
			preg_match('#Firefox/(\d{1,2})\.(\d{1,2})(\.(\d{1,2})(\.(\d{1,2}))?)?#i', $userAgent, $versionMatch);
			isset($versionMatch[1]) && $version[0] = (int)$versionMatch[1];
			isset($versionMatch[2]) && $version[1] = (int)$versionMatch[2];
			isset($versionMatch[4]) && $version[2] = (int)$versionMatch[4];
			isset($versionMatch[6]) && $version[3] = (int)$versionMatch[6];
		}
		else if (false !== strpos($userAgent, 'MSIE '))
		{
			//http://www.useragentstring.com/pages/Internet%20Explorer/
			$name = 'Internet Explorer';
			preg_match('#MSIE (\d{1,2})\.(\d{1,2})#i', $userAgent, $versionMatch);
			isset($versionMatch[1]) && $version[0] = (int)$versionMatch[1];
			isset($versionMatch[2]) && $version[1] = (int)$versionMatch[2];
		}
		else if (false !== strpos($userAgent, 'Iceweasel/')) // Firefox in Debian
		{
			// http://www.useragentstring.com/pages/Iceweasel/
			$name = 'Firefox'; //Iceweasel is identical to Firefox! no need to differt them
			preg_match('#Iceweasel/(\d{1,2})\.(\d{1,2})(\.(\d{1,2})(\.(\d{1,2}))?)?#i', $userAgent, $versionMatch);
			isset($versionMatch[1]) && $version[0] = (int)$versionMatch[1];
			isset($versionMatch[2]) && $version[1] = (int)$versionMatch[2];
			isset($versionMatch[4]) && $version[2] = (int)$versionMatch[4];
			isset($versionMatch[6]) && $version[3] = (int)$versionMatch[6];
		}
		else if (false !== strpos($userAgent, 'Chrome/'))
		{
			// http://www.useragentstring.com/pages/Chrome/
			$name = 'Chrome';
			preg_match('#Chrome/(\d{1,2})\.(\d{1,3})\.(\d{1,3}).(\d{1,3})#i', $userAgent, $versionMatch);
			isset($versionMatch[1]) && $version[0] = (int)$versionMatch[1];
			isset($versionMatch[2]) && $version[1] = (int)$versionMatch[2];
			isset($versionMatch[3]) && $version[2] = (int)$versionMatch[3];
			isset($versionMatch[4]) && $version[3] = (int)$versionMatch[4];
		}
		else if (false !== strpos($userAgent, 'Safari/'))
		{
			// http://www.useragentstring.com/pages/Safari/
			$name = 'Safari';
			/* Uncomment this block of code if u want to use Version/ tag
			 * instead of Safari/Build tag. Old Safari browsers haven’t Version/ tag
			 * and their version was marked as build number (ex. 528.16).
			if (false !== strpos($userAgent, 'Version/')) // old versions of Safari doesn't have Version tag in UserAgent
			{
				preg_match('#Version/(\d{1,2})\.(\d{1,2})(\.(\d{1,2}))?#i', $userAgent, $versionMatch);
				isset($versionMatch[1]) && $version[0] = (int)$versionMatch[1];
				isset($versionMatch[2]) && $version[1] = (int)$versionMatch[2];
				isset($versionMatch[4]) && $version[2] = (int)$versionMatch[4];
			}
			else
			{*/
				preg_match('#Safari/(\d{1,3})\.(\d{1,2})(\.(\d{1,2}))?#i', $userAgent, $versionMatch);
				isset($versionMatch[1]) && $version[0] = (int)$versionMatch[1];
				isset($versionMatch[2]) && $version[1] = (int)$versionMatch[2];
				isset($versionMatch[4]) && $version[2] = (int)$versionMatch[4];
			//}
		}
		
		return array('name' => $name, 'version' => $version);
	}
	
	/**
	 * Compare browser versions.
	 * 
	 * Returns int(0)  if version is aqual to $conditions,
	 *         int(-1) if version is older than $conditions,
	 *         int(1)  if version is newer than $conditions.
	 * 
	 * Returns NULL in case of any error.
	 * 
	 * @author Leontyev Valera (feedbee@gmail.com)
	 * @copyright 2009
	 * @license BSD
	 * 
	 * @param array $browser -- result of self::detectBrowser() method
	 * @param $conditions    -- vetsions to compre array('Opera' => array(9, 4), 'Firefox' => array(3, 1, 1), ...)
	 * @return int
	 */
	static public function checkForBrowserVersion(array $browser, array $conditions)
	{
		if (!isset($browser['name']) || !isset($conditions[$browser['name']])
			|| !isset($browser['version']) || count($browser['version']) < 1)
		{
			return null;
		}
		
		$cnd = $conditions[$browser['name']]; // 0=>, 1=>, 2=>
		if (!is_array($cnd))
		{
			return null;
		}
		
		for ($i = 0; $i < count($cnd); $i++)
		{
			if ($browser['version'][$i] < $cnd[$i])
			{
				return -1;
			}
			else if ($browser['version'][$i] > $cnd[$i])
			{
				return 1;
			}
		}
		
		return 0;
	}
}